/*
 * Copyright (c) 2016, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of Intel Corporation nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef CR_MGMT_GETSYSTEM_H
#define CR_MGMT_GETSYSTEM_H

#include <string>
#include <LogEnterExit.h>
#include <core/Result.h>
#include <core/NvmLibrary.h>
#include <core/exceptions/NoMemoryException.h>
#include <core/exceptions/LibraryException.h>
#include <persistence/config_settings.h>
#include <persistence/lib_persistence.h>
#include "SystemInfo.h"
#include "SystemMemoryResources.h"
#include "SoftwareInfo.h"
#include "SystemSocket.h"

namespace core
{
namespace system
{
class NVM_API SystemService
{
public:
	NVM_API SystemService(NvmLibrary &lib = NvmLibrary::getNvmLibrary());
	virtual ~SystemService();
	static SystemService &getService();

	virtual std::string getHostName();
	virtual Result<SystemInfo> getHostInfo();
	virtual Result<SystemMemoryResources> getMemoryResources();
	virtual Result<SoftwareInfo> getSoftwareInfo();
	virtual std::vector<NVM_UINT16> getAllSocketIds();
	virtual SystemSocketCollection getSocketsForSocketIds(const std::vector<NVM_UINT16> &socketIds);

protected:
	NvmLibrary &m_lib;
};

}
}

#endif //CR_MGMT_GETSYSTEM_H
